package jasarus.di.api;

import jasarus.di.spi.DependenciesRegistry;

import java.util.ServiceLoader;

/**
 * Static dependency injection utility.
 */
public final class Inject {

    // Static collaborators

    private final static DependenciesRegistry DEPENDENCIES_REGISTRY = resolveDependenciesRegistry();

    // Hidden constructor

    private Inject() {
    }

    // Injection methods

    public static <T> T inject(Class<T> type, String qualifier) {
        return DEPENDENCIES_REGISTRY.lookup(type, qualifier);
    }

    public static <T> T inject(Class<T> type) {
        return DEPENDENCIES_REGISTRY.lookup(type);
    }

    public static void injectInto(Object autowireCandidate) {
        DEPENDENCIES_REGISTRY.injectInto(autowireCandidate);
    }

    // Private helpers

    private static DependenciesRegistry resolveDependenciesRegistry() {
        final DependenciesRegistry registry = ServiceLoader.load(DependenciesRegistry.class).iterator().next();

        Runtime.getRuntime().addShutdownHook(new Thread() {
            @Override
            public void run() {
                registry.shutdown();
            }
        });
        return registry;
    }

}